'' ******************************************************************************
'' * James Burrows                                                              *
'' * Video diagnostics, or a poor mans O-scope!!                                *
'' ******************************************************************************
''
'' this object provides the PUBLIC functions:
''  -> Start
''
'' this object provides the PRIVATE functions:
''  -> None
''
'' this object uses the following sub OBJECTS:
''  -> DEBUG_PC
''  
CON
  _clkmode       = xtal1 + pll16x
  _xinfreq       = 5_000_000
  
  ' debug - Serial pins onboard pins (RX=31,TX=30 are the programming pins)
  pcDebugRX      = 31
  pcDebugTX      = 30

  ' serial baud rates  
  pcDebugBaud    = 256000  ' 57600, 115200 or 256000 only
    
VAR
  long s, ms, us
  byte VideoDiagnosticCogID
  long diagnosticDataArray[600]
  long baseaddress
  

OBJ
  debug           : "Debug_PC"

PUB Start
  s := clkfreq                              ' Clock cycles for 1 s
  ms:= clkfreq / 1_000                      ' Clock cycles for 1 ms
  us:= clkfreq / 1_000_000                  ' Clock cycles for 1 us

  ' start the PC debug object
  debug.startx(pcDebugRX,pcDebugTX,pcDebugBaud)
   
  VideoDiagnosticCogID := cognew(@entry,@diagnosticDataArray)

  repeat
    debug.str(string("FrameTime:"))
    debug.dec(long[@diagnosticDataArray][0]/ms)
    debug.str(string("ms  V-PulseLen:"))
    debug.dec(long[@diagnosticDataArray][1]/us)    
    debug.str(string("us  Lines:"))    
    debug.dec(long[@diagnosticDataArray][2])
    debug.str(string("  Lines in V-pulse:"))    
    debug.dec(long[@diagnosticDataArray][3])    
    debug.putc(13)

    debug.str(string("Line-PulseLengths:"))
    baseaddress := 3
    repeat 400
      debug.dec(long[@diagnosticDataArray][baseaddress]/us)
      debug.putc(",")      
      baseaddress++
    debug.putc(13)
      
    waitcnt(clkfreq*5+cnt)
 
    

dat
                        org

entry                   mov     VsyncPin,#1                 ' set Bit
                        shl     VsyncPin,#14                ' create PinMask
                        andn    dira,VsyncPin               ' ensure input

                        mov     CsyncPin,#1                 ' set Bit
                        shl     CsyncPin,#15                ' create PinMask
                        andn    dira,CsyncPin               ' ensure input
                                                                                      
goagain                 mov     FrameLinesCount,#0          ' pre setup
                        mov     FrameLinesInVPulseCount,#0  
                        mov     t1_ptr,par
                        add     t1_ptr,#16
                        
                        waitpeq zero,vsyncPin               ' wait for V sync LOW
                        mov     FrameStartCnt,cnt           ' start of frame/field

:countLines1            waitpeq zero,csyncpin               ' count lines and time between the V sync pulse
                        mov     LineStartCnt,cnt                                                
                        waitpeq csyncpin,csyncpin           ' wait high
                        waitpeq zero,csyncpin               ' wait low                                                                    
                        mov     LineEndCnt,cnt
                        subs    LineEndCnt,LineStartCnt     ' work out time
                        wrlong  LineEndCnt,t1_ptr           ' write time to long array
                        add     t1_ptr,#4
                        add     framelinescount,#1          ' add to frame count
                        add     FrameLinesInVPulseCount,#1

                        test    vsyncPin,ina          wc    ' check for end-of-field
              if_c      jmp     #:nextstep                  ' if V-Pin high then jump to :nextstep
                        jmp     #:countLines1               ' if V-Pin low then loop again

:nextstep               mov     FrameVPulseEndCnt,cnt       ' frame v sync pulse length
                        
:countLines2            waitpeq zero,csyncpin               ' count lines and time for each line in the
                        mov     LineStartCnt,cnt            ' rest of the frame/field.                                                
                        waitpeq csyncpin,csyncpin           ' wait high
                        waitpeq zero,csyncpin               ' wait low                                                                    
                        mov     LineEndCnt,cnt
                        subs    LineEndCnt,LineStartCnt     ' work out time
                        wrlong  LineEndCnt,t1_ptr           ' write time
                        add     t1_ptr,#4
                        add     framelinescount,#1          ' add to frame count

                        test    vsyncPin,ina          wc    ' check for end-of-field
              if_nc     jmp     #:FrameEnd                  ' if v-sync in low then end of frame/field
                        jmp     #:countLines2               ' else loop again 
                        
:FrameEnd               mov     FrameEndCnt,cnt             ' get the frame/field time
                        subs    FrameEndCnt,FrameStartCnt             ' work frame time out (clocks)
                        subs    FrameVPulseEndCnt,FrameStartCnt       ' work v-pulse length time out (clocks) 
                        
                        mov     t1_ptr,par                  ' write the frame time to long array
                        wrlong  FrameEndCnt,t1_ptr          
                        add     t1_ptr,#4                   ' write the frame v-pulse time to long array
                        wrlong  FrameVPulseEndCnt,t1_ptr                                                
                        add     t1_ptr,#4
                        wrlong  framelinescount,t1_ptr      ' write the line countto long array
                        add     t1_ptr,#4
                        wrlong  FrameLinesInVPulseCount,t1_ptr      ' write the line countto long array                        

                        waitpeq vsyncPin,vsyncPin           ' wait for V sync HIGH to get a clean frame next time
                        
                        jmp  #goagain                       ' loop


' Initialized data
VsyncPin                long 0
CSyncPin                long 0
FrameStartCnt           long 0
FrameEndCnt             long 0
FrameVPulseEndCnt       long 0
FrameLinesCount         long 0
FrameLinesInVPulseCount long 0
LineStartCnt            long 0
LineEndCnt              long 0
zero                    long 0
t1_ptr                  long 0